@@ -72,6 +72,8 @@ __attribute__((regparm(3)));
 #endif
 
 extern char * getVersion(char * progname, int maj, int min, int submin, int betavers);
+extern int getmac(char * macAddress, int strict, unsigned char * mac);
+
 
 #define BROADCAST "\xFF\xFF\xFF\xFF\xFF\xFF"
 
@@ -88,12 +90,12 @@ struct options
 	int bssid_set;				 /* bssid set flag       */
 	char essid[33];				 /* target ESSID         */
 	uchar bssid[6];				 /* target BSSID         */
-	int nbcpu;
-	/* # of cracker threads
-										(= # of CPU)*/
+	int nbcpu;					 /* # of cracker threads
+									(= # of CPU)         */
 	int is_quiet;				 /* quiet mode flag      */
 
 	uchar debug[64];			 /* user-defined WEP key */
+	int debug_row[64] ;          /* user-defined Row WEP key */
 	uchar maddr[6];				 /* MAC address filter   */
 	int keylen;					 /* WEP key length       */
 	int index;					 /* WEP key index        */
@@ -105,14 +107,15 @@ struct options
 	int is_bcdonly;				 /* binary coded decimal */
 
 	int do_brute;				 /* bruteforce last 2 KB */
-	int do_mt_brute;			 /* bruteforce last 2 KB multithreaded for SMP */
+	int do_mt_brute;			 /* bruteforce last 2 KB
+									multithreaded for SMP*/
 	int do_testy;				 /* experimental attack  */
 
 	FILE *dict;					 /* dictionary file      */
 	int no_stdin;				 /* if dict == stdin     */
 
 	int showASCII;				 /* Show ASCII version of*/
-	/* the wepkey           */
+								 /* the wepkey           */
 
 	int l33t;					 /* no comment           */
 }
@@ -253,7 +256,7 @@ const uchar R[256] =
 	, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255
 };
 
-char usage[] = 
+char usage[] =
 "\n"
 "  %s - (C) 2006 Thomas d\'Otreppe\n"
 "  Original work: Christophe Devine\n"
@@ -268,13 +271,13 @@ char usage[] =
 "      -b <bssid> : target selection: access point's MAC"
 "%s"
 "      -q         : enable quiet mode (no status output)\n"
-"\n" 
+"\n"
 "  Static WEP cracking options:\n"
 "\n"
 "      -c         : search alpha-numeric characters only\n"
 "      -t         : search binary coded decimal chr only\n"
 "      -h         : search the numeric key for Fritz!BOX\n"
-"      -d <start> : debug - specify beginning of the key\n"
+"      -d <mask>  : debug - specify mask of the key (A1:XX:CF)\n"
 "      -m <maddr> : MAC address to filter usable packets\n"
 "      -n <nbits> : WEP key length :  64/128/152/256/512\n"
 "      -i <index> : WEP key index (1 to 4), default: any\n"
@@ -1394,8 +1397,8 @@ int check_wep_key( uchar *wepkey, int B, int keylen )
 		printf( "(ASCII: " );
 
 		for( i = 0; i < opt.keylen; i++ )
-			printf( "%c", ( ( wep.key[i] >  31 && wep.key[i] < 127 ) ||
-				wep.key[i] > 160 ) ? wep.key[i] : '.' );
+			printf( "%c", ( ( wepkey[i] >  31 && wepkey[i] < 127 ) ||
+				wepkey[i] > 160 ) ? wepkey[i] : '.' );
 
 		printf( " )" );
 	}
@@ -1468,8 +1471,8 @@ int calc_poll( int B )
 
 	/* set votes to the max if the keybyte is user-defined */
 
-	if( B < opt.debug[0] )
-		wep.poll[B][opt.debug[1 + B]].val = 32767;
+	if( opt.debug_row[B] )
+		wep.poll[B][opt.debug[B]].val = 32767;
 
 	/* if option is set, restrict keyspace to alpha-numeric */
 
@@ -1842,7 +1845,8 @@ int do_wep_crack2( int B )
 
 int inner_bruteforcer_thread(void *arg)
 {
-	int i, j, nthread = (int)arg;
+	int i, j;
+	size_t nthread = (size_t)arg;
 	uchar wepkey[64];
 
 	inner_bruteforcer_thread_start:
@@ -2167,7 +2171,7 @@ uchar mic[16], int force )
 {
 	float delta;
 	int i, et_h, et_m, et_s;
-	char tmpbuf[27];
+	char tmpbuf[28];
 
 	#ifdef __i386__
 	__asm__( "emms" );			 /* clean up the fp regs */
@@ -2487,9 +2491,9 @@ int main( int argc, char *argv[] )
 	{
 
 		if ( max_cpu == 1 )
-			option = getopt( argc, argv, "a:e:b:qcthd:m:n:i:f:k:x::yw:0" );
+			option = getopt( argc, argv, "a:e:b:qcthd:m:n:i:f:k:x::ysw:0" );
 		else
-			option = getopt( argc, argv, "a:e:b:p:qcthd:m:n:i:f:k:x::Xyw:0" );
+			option = getopt( argc, argv, "a:e:b:p:qcthd:m:n:i:f:k:x::Xysw:0" );
 
 		if( option < 0 ) break;
 
@@ -2522,31 +2526,10 @@ int main( int argc, char *argv[] )
 
 			case 'b' :
 
-				i = 0;
-				s = optarg;
-
-				while( sscanf( s, "%x", &n ) == 1 )
+				if (getmac(optarg, 1, opt.bssid) != 0)
 				{
-					if( n < 0 || n > 255 )
-					{
 						printf( "Invalid BSSID (not a MAC).\n" );
 						return( FAILURE );
-					}
-
-					opt.bssid[i] = n;
-
-					if( ++i > 6 ) break;
-
-					if( ! ( s = strchr( s, ':' ) ) )
-						break;
-
-					s++;
-				}
-
-				if( i != 6 )
-				{
-					printf( "Invalid BSSID (not a MAC).\n" );
-					return( FAILURE );
 				}
 
 				opt.bssid_set = 1;
@@ -2585,70 +2568,47 @@ int main( int argc, char *argv[] )
 
 			case 'd' :
 
-				i = 1;
+				i = 0 ;
+				n = 0;
 				s = optarg;
-
+				while( s[i] != '\0' )
+				{
+					if (s[i] == 'x')
+						s[i] = 'X';
+					if ( s[i] == '-' ||  s[i] == ':' || s[i] == ' ')
+						i++;
+					else
+						s[n++] = s[i++];
+				}
+				s[n] = '\0' ;
 				buf[0] = s[0];
 				buf[1] = s[1];
 				buf[2] = '\0';
-
-				while( sscanf( buf, "%x", &n ) == 1 )
+				i = 0;
+				while( ( sscanf( buf, "%x", &n ) == 1 ) || ( buf[0] == 'X' && buf[1] == 'X' ) )
 				{
-					if( n < 0 || n > 255 )
-					{
-						printf( "Invalid debug key.\n" );
-						return( FAILURE );
+					if ( buf[0] == 'X' && buf[1] == 'X' ) {
+						opt.debug_row[i++] = 0 ;
+					} else {
+						if ( n < 0 || n > 255 )
+						{
+							printf( "Invalid debug key.\n" );
+							return( FAILURE );
+						}
+						opt.debug[i] = n ;
+						opt.debug_row[i++] = 1;
 					}
-
-					opt.debug[i++] = n;
-
-					if( i >= 62 ) break;
-
+					if( i >= 64 ) break;
 					s += 2;
-
-					if( s[0] == ':' || s[0] == '-' )
-						s++;
-
-					if( s[0] == '\0' || s[1] == '\0' )
-						break;
-
 					buf[0] = s[0];
 					buf[1] = s[1];
 				}
-
-				if( i == 0 )
-				{
-					printf( "Invalid debug key.\n" );
-					return( FAILURE );
-				}
-
-				opt.debug[0] = i - 1;
 				break;
 
+
 			case 'm' :
 
-				i = 0;
-				s = optarg;
-
-				while( sscanf( s, "%x", &n ) == 1 )
-				{
-					if( n < 0 || n > 255 )
-					{
-						printf( "Invalid MAC address filter.\n" );
-						return( FAILURE );
-					}
-
-					opt.maddr[i] = n;
-
-					if( ++i > 6 ) break;
-
-					if( ! ( s = strchr( s, ':' ) ) )
-						break;
-
-					s++;
-				}
-
-				if( i != 6 )
+				if ( getmac(optarg, 1, opt.maddr) != 0)
 				{
 					printf( "Invalid MAC address filter.\n" );
 					return( FAILURE );
