@@ -26,7 +26,6 @@
 #include <time.h>
 
 #include "version.h"
-#include "crctable.h"
 #include "crypto.h"
 #include "pcap.h"
 
@@ -35,6 +34,8 @@
 #define CRYPT_WPA  2
 
 extern char * getVersion(char * progname, int maj, int min, int submin, int betavers);
+extern int check_crc_buf( unsigned char *buf, int len );
+extern int calc_crc_buf( unsigned char *buf, int len );
 
 const short Sbox[2][256]=
 {
@@ -117,12 +118,12 @@ char usage[] =
 "\n"
 "  usage: airdecap-ng [options] <pcap file>\n"
 "\n"
-"      -l       : don't remove the 802.11 header\n"
-"      -b bssid : access point MAC address filter\n"
-"      -k pmk   : WPA Pairwise Master Key in hex\n"
-"      -e essid : target network ascii identifier\n"
-"      -p pass  : target network WPA passphrase\n"
-"      -w key   : target network WEP key in hex\n"
+"      -l         : don't remove the 802.11 header\n"
+"      -b <bssid> : access point MAC address filter\n"
+"      -k <pmk>   : WPA Pairwise Master Key in hex\n"
+"      -e <essid> : target network SSID\n"
+"      -p <pass>  : target network WPA passphrase\n"
+"      -w <key>   : target network WEP key in hex\n"
 "\n";
 
 
@@ -260,34 +261,6 @@ int calc_ptk( struct ST_info *wpa, uchar pmk[32] )
     return( memcmp( mic, wpa->keymic, 16 ) == 0 );
 }
 
-/* CRC checksum calculation routine */
-
-int calc_crc_buf( unsigned char *buf, int len )
-{
-    unsigned long crc = 0xFFFFFFFF;
-
-    for( ; len > 0; len--, buf++ )
-        crc = crc_tbl[(crc ^ *buf) & 0xFF] ^ ( crc >> 8 );
-
-    return( ~crc );
-}
-
-/* CRC checksum verification routine */
-
-int check_crc_buf( unsigned char *buf, int len )
-{
-    unsigned long crc = 0xFFFFFFFF;
-
-    for( ; len > 0; len--, buf++ )
-        crc = crc_tbl[(crc ^ *buf) & 0xFF] ^ ( crc >> 8 );
-
-    crc = ~crc;
-
-    return( ( ( crc       ) & 0xFF ) == buf[0] &&
-            ( ( crc >>  8 ) & 0xFF ) == buf[1] &&
-            ( ( crc >> 16 ) & 0xFF ) == buf[2] &&
-            ( ( crc >> 24 ) & 0xFF ) == buf[3] );
-}
 
 /* WEP (barebone RC4) decryption routine */
 
@@ -578,7 +551,7 @@ int main( int argc, char *argv[] )
     struct ST_info *st_cur;
     struct ST_info *st_prv;
     struct pcap_file_header pfh;
-    struct pcap_pkthdr pkh;    
+    struct pcap_pkthdr pkh;
 
     /* parse the arguments */
 
@@ -680,6 +653,12 @@ int main( int argc, char *argv[] )
 
             case 'e' :
 
+				if ( opt.essid[0])
+				{
+					printf( "ESSID already specified.\n" );
+                    return( 1 );
+				}
+
                 memset(  opt.essid, 0, sizeof( opt.essid ) );
                 strncpy( opt.essid, optarg, sizeof( opt.essid ) - 1 );
                 break;
@@ -854,7 +833,7 @@ int main( int argc, char *argv[] )
     pfh.thiszone        = 0;
     pfh.sigfigs         = 0;
     pfh.snaplen         = 65535;
-    pfh.linktype        = ( opt.no_convert ) ? 
+    pfh.linktype        = ( opt.no_convert ) ?
                             LINKTYPE_IEEE802_11 :
                             LINKTYPE_ETHERNET;
 
@@ -972,7 +951,7 @@ int main( int argc, char *argv[] )
 
         if( memcmp( opt.bssid, ZERO, 6 ) != 0 )
             if( memcmp( opt.bssid, bssid, 6 ) != 0 )
-                continue;    
+                continue;
 
         /* locate the station's MAC address */
 
