@@ -65,17 +65,19 @@
 #define NB_PRB 10       /* size of probed ESSID ring buffer */
 
 extern char * getVersion(char * progname, int maj, int min, int submin, int betavers);
-extern int is_ndiswrapper(const char * iface);
-extern int iwpriv_exist(void);
+extern int is_ndiswrapper(const char * iface, const char * path);
+extern char * wiToolsPath(const char * tool);
 
+const unsigned char llcnull[4] = {0, 0, 0, 0};
 char *f_ext[4] = { "txt", "gps", "cap", "ivs" };
 
-int abg_chans [] = 
+int abg_chans [] =
 {
     1, 7, 13, 2, 8, 3, 14, 9, 4, 10, 5, 11, 6, 12,
     36, 40, 44, 48, 52, 56, 60, 64, 100, 104, 108,
     112, 116, 120, 124, 128, 132, 136, 140, 149,
-    153,157,161,184,188,192,196,200,204,208,212,216,0
+    153, 157, 161, 184, 188, 192, 196, 200, 204,
+    208, 212, 216,0
 };
 
 int bg_chans  [] =
@@ -87,7 +89,8 @@ int a_chans   [] =
 {
     36, 40, 44, 48, 52, 56, 60, 64, 100, 104, 108,
     112, 116, 120, 124, 128, 132, 136, 140, 149,
-    153,157,161,184,188,192,196,200,204,208,212,216,0
+    153, 157, 161, 184, 188, 192, 196, 200, 204,
+    208, 212, 216,0
 };
 
 /* linked list of detected access points */
@@ -166,12 +169,16 @@ struct globals
     int is_madwifing;       /* set if madwifi-ng    */
     int do_exit;            /* interrupt flag       */
     struct winsize ws;      /* console window size  */
-    
+
     char * elapsed_time;	/* capture time			*/
-    
+
     int one_beacon;         /* Record only 1 beacon?*/
-    
+
     int record_data;		/* do we record data?   */
+
+    char * iwpriv;
+    char * wlanctlng;
+    char * wl;
 }
 G;
 
@@ -188,16 +195,14 @@ char usage[] =
 "  Options:\n"
 "      --ivs               : Save only captured IVs\n"
 "      --gpsd              : Use GPSd\n"
-"      --write    <prefix> : Dump file prefix (mandatory)\n"
+"      --write    <prefix> : Dump file prefix\n"
 "      -w                  : same as --write \n"
 "      --beacons           : Record all beacons in dump file\n"
 "\n"
 "  By default, airodump-ng hop on 2.4Ghz channels.\n"
 "  You can make it capture on other/specific channel(s) by using:\n"
 "      --channel <channel> : Capture on a specific channel\n"
-"      -c                  : Same as --channel\n"
-"      -a                  : Will hop on 5Ghz channels\n"
-"      --abg               : Will hop on 2.4 and 5Ghz channels\n"
+"      --band <abg>        : Band on which airodump-ng should hop\n"
 "\n";
 
 
@@ -211,9 +216,9 @@ int dump_initialize( char *prefix, int ivs_only )
 
 
 	/* If you only want to see what happening, send all data to /dev/null */
-	
+
 	if ( prefix == NULL) {
-		return( 0 );	
+		return( 0 );
 	}
 
     /* check not to overflow the ofn buffer */
@@ -323,11 +328,11 @@ int dump_initialize( char *prefix, int ivs_only )
 int dump_add_packet( unsigned char *h80211, int caplen, int power )
 {
     int i, n, z;
+    struct pcap_pkthdr pkh;
+    struct timeval tv;
     unsigned char *p, c;
     unsigned char bssid[6];
     unsigned char stmac[6];
-    struct pcap_pkthdr pkh;
-    struct timeval tv;
 
     struct AP_info *ap_cur = NULL;
     struct ST_info *st_cur = NULL;
@@ -344,6 +349,12 @@ int dump_add_packet( unsigned char *h80211, int caplen, int power )
     if( ( h80211[0] & 0x0C ) == 0x04 )
         goto write_packet;
 
+	/* if it's a LLC null packet, just forget it (may change in the future) */
+
+	if ( caplen > 28)
+		if ( memcmp(h80211 + 24, llcnull, 4) == 0)
+			return ( 0 );
+
     /* locate the access point's MAC address */
 
     switch( h80211[1] & 3 )
@@ -445,6 +456,7 @@ int dump_add_packet( unsigned char *h80211, int caplen, int power )
 
     ap_cur->nb_pkt++;
 
+
     /* locate the station MAC in the 802.11 header */
 
     switch( h80211[1] & 3 )
@@ -534,7 +546,7 @@ int dump_add_packet( unsigned char *h80211, int caplen, int power )
 
     st_cur->tlast = time( NULL );
 
-    /* only update power if packets comes from the 
+    /* only update power if packets comes from the
      * client: either type == Mgmt and SA != BSSID,
      * or FromDS == 0 and ToDS == 1 */
 
@@ -929,7 +941,7 @@ int getBatteryState()
     int flag;
     char units[32];
 
-    if (linux_apm == 1) 
+    if (linux_apm == 1)
     {
         if ((apm = fopen("/proc/apm", "r")) != NULL ) {
         	if ( fgets(buf, 128,apm) != NULL ) {
@@ -1002,9 +1014,9 @@ int getBatteryState()
         while (batteries != NULL && ((this_battery = readdir(batteries)) != NULL)) {
             if (this_battery->d_name[0] == '.')
                 continue;
-            
+
             snprintf(battery_info, sizeof(battery_info), "/proc/acpi/battery/%s/info", this_battery->d_name);
-            info = fopen(battery_info, "r");    
+            info = fopen(battery_info, "r");
             batt_full_capacity[batno] = 0;
             if ( info != NULL ) {
                 while (fgets(buf, sizeof(buf), info) != NULL)
@@ -1012,8 +1024,8 @@ int getBatteryState()
                         continue;
                 fclose(info);
             }
-    
-                
+
+
             snprintf(battery_state, sizeof(battery_state),
                 "/proc/acpi/battery/%s/state", this_battery->d_name);
             if ((acpi = fopen(battery_state, "r")) == NULL)
@@ -1081,7 +1093,7 @@ char * getStringTimeFromSec(double seconds)
     char * HourTime;
     char * MinTime;
 
-	if (seconds <0) 
+	if (seconds <0)
 		return NULL;
 
 	ret = (char *) calloc(1,256);
@@ -1106,13 +1118,13 @@ char * getStringTimeFromSec(double seconds)
     {
     	if (hour[2] == 0 && hour[1] == 0)
     		sprintf(ret, "%d s", hour[0] );
-        else 
+        else
         	sprintf(ret, "%s", (hour[2] == 0) ? MinTime : HourTime );
 	}
-	
+
     free(MinTime);
     free(HourTime);
-			
+
 	return ret;
 
 }
@@ -1133,7 +1145,7 @@ char * getBatteryString(void)
     }
 
 	batt_string = getStringTimeFromSec( (double) batt_time );
-	
+
 	ret = (char *) calloc( 1, 256 );
 
 	sprintf( ret,"][ BAT: %s ]", batt_string );
@@ -1162,15 +1174,15 @@ void dump_print( int ws_row, int ws_col )
     tt = time( NULL );
     lt = localtime( &tt );
 
-    /* 
-     *  display the channel, battery, position (if we are connected to GPSd) 
+    /*
+     *  display the channel, battery, position (if we are connected to GPSd)
      *  and current time
      */
 
     memset( strbuf, ' ', ws_col - 1 );
     strbuf[ws_col - 1] = '\0';
     fprintf( stderr, "%s\n", strbuf );
-    
+
 
     if (G.gps_loc[0]) {
         snprintf( strbuf, sizeof( strbuf ) - 1,
@@ -1180,7 +1192,7 @@ void dump_print( int ws_row, int ws_col )
               G.elapsed_time , 1900 + lt->tm_year,
               1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min );
     }
-    else 
+    else
     {
         snprintf( strbuf, sizeof( strbuf ) - 1,
               " CH %2d %s[ Elapsed: %s ][ %04d-%02d-%02d %02d:%02d ",
@@ -1377,7 +1389,7 @@ int dump_write_csv( void )
 
     if (! G.record_data)
     	return 0;
-    
+
     fseek( G.f_txt, 0, SEEK_SET );
 
     fprintf( G.f_txt,
@@ -1525,7 +1537,7 @@ void gps_tracker( void )
     int gpsd_sock;
     char line[256], *p;
     struct sockaddr_in gpsd_addr;
-    
+
     /* attempt to connect to localhost, port 2947 */
 
     gpsd_sock = socket( AF_INET, SOCK_STREAM, 0 );
@@ -1533,7 +1545,7 @@ void gps_tracker( void )
     if( gpsd_sock < 0 ) {
         return;
     }
-    
+
     gpsd_addr.sin_family      = AF_INET;
     gpsd_addr.sin_port        = htons( 2947 );
     gpsd_addr.sin_addr.s_addr = inet_addr( "127.0.0.1" );
@@ -1661,7 +1673,7 @@ int set_channel( char *interface, int fd_raw, int channel )
         if( ( pid = fork() ) == 0 )
         {
             close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "wlanctl-ng", "wlanctl-ng", interface,
+            execl( G.wlanctlng, "wlanctl-ng", interface,
                     "lnxreq_wlansniff", s, NULL );
             exit( 1 );
         }
@@ -1681,7 +1693,7 @@ int set_channel( char *interface, int fd_raw, int channel )
         if( ( pid = fork() ) == 0 )
         {
             close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "iwpriv", "iwpriv", interface,
+            execlp( G.iwpriv, "iwpriv", interface,
                     "monitor", "1", s, NULL );
             exit( 1 );
         }
@@ -1710,33 +1722,42 @@ int set_channel( char *interface, int fd_raw, int channel )
 
 int set_monitor( char *interface, int fd_raw )
 {
-    int pid, status;
+	char s[32];
+    int pid, status, channel;
     struct iwreq wrq;
 
+	channel = (G.channel == 0 ) ? 10 : G.channel;
+
 	if( strcmp(interface,"prism0") == 0 )
 	{
 		if( ( pid = fork() ) == 0 )
 		{
 			close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-			execlp( "wl", "wl", "monitor", "1", NULL);
+			execl( G.wl, "wl", "monitor", "1", NULL);
 			exit( 1 );
 		}
 		waitpid( pid, &status, 0 );
 		if( WIFEXITED(status) )
 			return( WEXITSTATUS(status) );
-		return( 1 );		
+		return( 1 );
+	}
+	else if (strncmp(interface, "rtap", 4) == 0 )
+	{
+		return 0;
 	}
 	else
 	{
 		if( G.is_wlanng )
 		{
+			snprintf( s,  sizeof( s ) - 1, "channel=%d", channel );
 			if( ( pid = fork() ) == 0 )
 			{
 				close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-				execlp( "wlanctl-ng", "wlanctl-ng", interface,
+				execl( G.wlanctlng, "wlanctl-ng", interface,
 						"lnxreq_wlansniff", "enable=true",
 						"prismheader=true", "wlanheader=false",
-						"stripfcs=true", "keepwepflags=true", NULL );
+						"stripfcs=true", "keepwepflags=true",
+						s, NULL );
 				exit( 1 );
 			}
 
@@ -1756,13 +1777,13 @@ int set_monitor( char *interface, int fd_raw )
 			perror( "ioctl(SIOCSIWMODE) failed" );
 			return( 1 );
 		}
-	
+
 	}
-	
+
 	/* Check later if it's really usefull to set channel here */
-	
-	set_channel( interface, fd_raw, (G.channel == 0 ) ? 10 : G.channel);
-	
+
+	set_channel( interface, fd_raw, channel);
+
     return( 0 );
 }
 
@@ -1778,7 +1799,7 @@ void channel_hopper( char *interface, int fd_raw )
         if( G.channels[ch_idx] ==  0 )
             ch_idx = 0;
 
-        if( G.channels[ch_idx] == -1 ) 
+        if( G.channels[ch_idx] == -1 )
             continue;
 
         ch = G.channels[ch_idx];
@@ -1799,10 +1820,11 @@ void channel_hopper( char *interface, int fd_raw )
 int main( int argc, char *argv[] )
 {
     long time_slept;
-    int n, caplen, pid;
+    int i, n, caplen, pid;
     int fd_raw, arptype;
     int ivs_only, power;
     int valid_channel, chanoption;
+    int freq [2];
     time_t tt1, tt2, tt3, start_time;
 
     unsigned char      *buffer;
@@ -1820,7 +1842,7 @@ int main( int argc, char *argv[] )
     /*
     struct sockaddr_in provis_addr;
     */
-    
+
     fd_set             rfds;
     FILE               *check_madwifing;
 
@@ -1846,7 +1868,7 @@ int main( int argc, char *argv[] )
 	G.f_ivs        =  NULL;
 	G.f_txt        =  NULL;
     G.f_gps        =  NULL;
-    
+
     #ifdef linux
         linux_acpi =  1;
         linux_apm  =  1;
@@ -1861,35 +1883,23 @@ int main( int argc, char *argv[] )
         return( 1 );
     }
 
-
     do
     {
         int option_index = 0;
-        
+
         static struct option long_options[] = {
-            {"ivs",     0, 0, 'i'},
-            {"channel", 1, 0, 'c'},
-            {"write",   1, 0, 'w'},
-            {"gpsd",    0, 0, 'p'},
-            {"ab",      0, 0, 'h'},
-            {"ag",      0, 0, 'h'},
-            {"ba",      0, 0, 'h'},
-            {"bg",      0, 0, 'b'},
-            {"ga",      0, 0, 'h'},
-            {"gb",      0, 0, 'b'},
-            {"abg",     0, 0, 'h'},
-            {"agb",     0, 0, 'h'},
-            {"bga",     0, 0, 'h'},
-            {"bag",     0, 0, 'h'},
-            {"gba",     0, 0, 'h'},
-            {"gab",     0, 0, 'h'},
-            {"hop",     0, 0, 'h'},
+			{"band",    1, 0, 'b'},
+			{"beacon",  0, 0, 'e'},
             {"beacons", 0, 0, 'e'},
+            {"channel", 1, 0, 'c'},
+            {"gpsd",    0, 0, 'g'},
+            {"ivs",     0, 0, 'i'},
+			{"write",   1, 0, 'w'},
             {0,         0, 0,  0 }
         };
 
         int option = getopt_long( argc, argv,
-                        "abc:giw:",
+                        "b:c:egiw:",
                         long_options, &option_index );
 
         if( option < 0 ) break;
@@ -1898,16 +1908,6 @@ int main( int argc, char *argv[] )
         {
             case 0 :
 
-                break;
-
-            case 'a' :
-				if (chanoption == 1) {
-					printf( "Notice: Channel range already given\n" );
-					break;
-				}
-					
-				chanoption = 1;
-                G.channels = a_chans;
                 break;
 
 			case 'e':
@@ -1923,9 +1923,9 @@ int main( int argc, char *argv[] )
 						printf( "Notice: Channel already given (%d)\n", G.channel);
 					break;
 				}
-				
+
                 sscanf(optarg, "%d",&(G.channel));
-                
+
                 if ( G.channel < 0 )
                     goto usage;
 
@@ -1943,29 +1943,38 @@ int main( int argc, char *argv[] )
                 }
                 /*
                  * else, hop on 2.4Ghz channels
-                 * so, no need to break
 				 */
+				G.channels = bg_chans;
+				break;
 
-            case 'g' :              
             case 'b' :
 
 				if (chanoption == 1 && option != 'c') {
 					printf( "Notice: Channel range already given\n" );
 					break;
 				}
+				freq[0] = freq[1] = 0;
 
-				/* Default behaviour: hop on 2.4Ghz channels */
-                G.channels = bg_chans;                      
-                break;
-
-            case 'h' :
-				if (chanoption == 1) {
-					printf( "Notice: Channel range already given\n" );
-					break;
+				for (i = 0; i < (int)strlen(optarg); i++) {
+					if ( optarg[i] == 'a' )
+						freq[1] = 1;
+					else if ( optarg[i] == 'b' || optarg[i] == 'g')
+						freq[0] = 1;
+					else {
+						printf( "Error: invalid band (%c)\n", optarg[i] );
+						exit ( 1 );
+					}
 				}
-                
-                /* hop on all available channels */
-                G.channels     =  abg_chans;
+
+				if (freq[1] + freq[0] == 2 )
+					G.channels = abg_chans;
+				else {
+					if ( freq[1] == 1 )
+						G.channels = a_chans;
+					else
+						G.channels = bg_chans;
+				}
+
                 break;
 
             case 'i':
@@ -1973,8 +1982,8 @@ int main( int argc, char *argv[] )
                 ivs_only = 1;
                 break;
 
-            case 'p':
-            
+            case 'g':
+
                 G.usegpsd  = 1;
                 /*
                 if (inet_aton(optarg, &provis_addr.sin_addr) == 0 )
@@ -2024,14 +2033,16 @@ int main( int argc, char *argv[] )
             fprintf( stderr, "This program requires root privileges.\n" );
         return( 1 );
     }
-    
+
     setuid( getuid() );
 
     /* Check iwpriv existence */
 
-    if (! iwpriv_exist() )
+	G.iwpriv = wiToolsPath("iwpriv");
+
+    if (! G.iwpriv )
     {
-        printf("Can't find wireless tools, exiting.\n");
+        fprintf(stderr, "Can't find wireless tools, exiting.\n");
         return (1);
     }
 
@@ -2057,12 +2068,15 @@ int main( int argc, char *argv[] )
     }
 
     /* Exit if ndiswrapper : check iwpriv ndis_reset */
-    
-    if ( is_ndiswrapper(iface) ) {
-    	printf("Ndiswrapper doesn't support monitor mode.\n");
+
+    if ( is_ndiswrapper(iface, G.iwpriv) ) {
+    	fprintf(stderr, "Ndiswrapper doesn't support monitor mode.\n");
     	return (1);
     }
 
+	if( strcmp(iface,"prism0") == 0 )
+		G.wl = wiToolsPath("wl");
+
     memset( &sll, 0, sizeof( sll ) );
     sll.sll_family   = AF_PACKET;
     sll.sll_ifindex  = ifr.ifr_ifindex;
@@ -2070,10 +2084,11 @@ int main( int argc, char *argv[] )
 
     if( memcmp( iface, "wlan", 4 ) == 0 )
     {
+		G.wlanctlng = wiToolsPath("wlanctl-ng");
         if( ( pid = fork() ) == 0 )     /* wlan-ng brain damage */
         {
             close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "wlanctl-ng", "wlanctl-ng", iface,
+            execl( G.wlanctlng, "wlanctl-ng", iface,
                     "lnxreq_ifstate", "ifstate=enable", NULL );
             exit( 1 );
         }
@@ -2086,7 +2101,7 @@ int main( int argc, char *argv[] )
         if( ! fork() )                  /* hostap card reset */
         {
             close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "iwpriv", "iwpriv", iface, "reset", "1", NULL );
+            execl( G.iwpriv, "iwpriv", iface, "reset", "1", NULL );
             exit( 1 );
         }
         wait( NULL );
@@ -2099,7 +2114,7 @@ int main( int argc, char *argv[] )
         if( ( pid = fork() ) == 0 )
         {
             close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "iwpriv", "iwpriv", iface, "get_port3", NULL );
+            execl( G.iwpriv, "iwpriv", iface, "get_port3", NULL );
             exit( 1 );
         }
 
@@ -2121,7 +2136,7 @@ int main( int argc, char *argv[] )
         G.is_madwifing = 1;
         memset(buffer,0, 65536);
         sprintf((char *) buffer, "/proc/sys/net/%s/dev_type", iface);
-        FILE * f = fopen(buffer,"w");
+        FILE * f = fopen((char*)buffer,"w");
         if (f != NULL) {
             char * madwifiPrism2 = "802\n";
             fprintf(f, madwifiPrism2);
@@ -2133,7 +2148,7 @@ int main( int argc, char *argv[] )
 
     /* make sure the interface is up */
 
-    ifr.ifr_flags = IFF_UP | IFF_BROADCAST | IFF_RUNNING;        
+    ifr.ifr_flags = IFF_UP | IFF_BROADCAST | IFF_RUNNING;
 
     if( ioctl( fd_raw, SIOCSIFFLAGS, &ifr ) < 0 )
     {
@@ -2141,7 +2156,7 @@ int main( int argc, char *argv[] )
         return( 1 );
     }
 
-    if (set_monitor( iface, fd_raw ))
+    if ( set_monitor( iface, fd_raw ) )
     {
         printf("Error setting monitor mode on %s\n",iface);
         return( 1 );
@@ -2161,7 +2176,7 @@ int main( int argc, char *argv[] )
     if( ! fork() )  /* hostap */
     {
         close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-        execlp( "iwpriv", "iwpriv", iface, "monitor_type", "1", NULL );
+        execlp( G.iwpriv, "iwpriv", iface, "monitor_type", "1", NULL );
         exit( 1 );
     }
     wait( NULL );
@@ -2169,7 +2184,7 @@ int main( int argc, char *argv[] )
     if( ! fork() )  /* r8180 */
     {
         close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-        execlp( "iwpriv", "iwpriv", iface, "prismhdr", "1", NULL );
+        execl( G.iwpriv, "iwpriv", iface, "prismhdr", "1", NULL );
         exit( 1 );
     }
     wait( NULL );
@@ -2177,14 +2192,23 @@ int main( int argc, char *argv[] )
     if( ! fork() )  /* prism54 */
     {
         close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-        execlp( "iwpriv", "iwpriv", iface, "set_prismhdr", "1", NULL );
+        execl( G.iwpriv, "iwpriv", iface, "set_prismhdr", "1", NULL );
         exit( 1 );
     }
     wait( NULL );
 
-    /* make sure the WEP key is off */
+    if( ! fork() )  /* rt2570-cvs */
+    {
+        close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
+        execlp( "iwpriv", "iwpriv", argv[1], "forceprismheader", "1", NULL );
+        exit( 1 );
+    }
+    wait( NULL );
 
-    disable_wep_key( iface, fd_raw );
+
+    /* make sure the WEP key is off only if it's not rtap0 */
+	if ( strncmp( iface, "rtap", 4) )
+    	disable_wep_key( iface, fd_raw );
 
     /* start a child to hop between channels */
 
@@ -2305,14 +2329,14 @@ int main( int argc, char *argv[] )
         {
             /* update the battery state */
 			free(G.batt);
-			
+
             tt2 			= time( NULL );
             G.batt 			= getBatteryString();
-            
+
             /* update elapsed time */
 
 			free(G.elapsed_time);
-    		G.elapsed_time = getStringTimeFromSec( 
+    		G.elapsed_time = getStringTimeFromSec(
     			difftime(tt2, start_time) );
 
 
@@ -2450,7 +2474,7 @@ int main( int argc, char *argv[] )
     	if( G.f_cap != NULL ) fclose( G.f_cap );
     	if( G.f_ivs != NULL ) fclose( G.f_ivs );
 	}
-	
+
     if( ! G.save_gps )
     {
         sprintf( (char *) buffer, "%s-%02d.gps", argv[2], G.f_index );
