@@ -261,7 +261,7 @@ int main( int argc, char *argv[] )
     buffer[1] = s[1];
     buffer[2] = '\0';
 
-    while( sscanf( buffer, "%x", &n ) == 1 )
+    while( sscanf( (char*) buffer, "%x", &n ) == 1 )
     {
         if( n < 0 || n > 255 )
         {
@@ -280,7 +280,7 @@ int main( int argc, char *argv[] )
 
         if( s[0] == '\0' || s[1] == '\0' )
             break;
-        
+
         buffer[0] = s[0];
         buffer[1] = s[1];
     }
